/*
Bone creation example using MaxScript

-- By ken turner
-- www.designimage.co.uk

This Script shows how to build a simple skeleton using arrays of positions.

*/

fn createBones nametxt posList parentBone:undefined BoneSize:0.2 = -- A function similar to this can be found in the longer 'wing rigger' script.
(
	-- poslist is an array of positions. The start and end points of each bone.
	-- To create the smaller 'end bone' of the chain, we'll add a new position to this array	
	-- We'll do this by creating a vector ( an [ x ,y ,z ] value ). Giving it the length 'BoneSize', and adding it to the last position in the array.
	
	v = posList[posList.count]  - posList[posList.count - 1] -- make a vector from the last two positions
	v = v /  ( length v ) -- normalise it (make it's length equal to one )
	endPoint = posList[posList.count] + ( v * BoneSize ) -- scale it by BoneSize and add to it the end
	append posList endPoint 
	
	--- now use the position array to create some bones ------- 
	BoneCount = ( posList.count - 1)
	myBoneArray = for i = 1 to BoneCount collect  -- 'collect' means 'make an array of'
	(
		theBone = BoneSys.createBone posList[i] posList[i+1] [0,0,1]  --- this actually creates the bone
		
		theBone.width = theBone.height = BoneSize
		
		theBone.parent = parentBone -- it's ok to set theBone.parent = undefined the first time round
		parentBone = theBone
		NameSuffix = if (i == BoneCount ) then "end" else (i as string) -- if this the last bone add "end" to its name.
		theBone.name = nametxt + "_bone_"+ NameSuffix
		theBone.rotation.controller = TCB_rotation()
		
		nodeInvalRect theBone -- make sure the max viewports are refreshed where the bone is created
		
		theBone -- return the bone for collection into 'myBoneArray'
	)
	
	-- return myBoneArray  -- we don't need to write 'return' because MAXscript automatically returns the last value used in a block of code.
)

armPos = #([0.38,-1.03,0], [0.93,-1.3,0], [2.61,0.39,0], [5.92,-1.14,0], [8.61,-0.81,0], [10.79,0.06,0])
bodyPos = #([0.612794,-0.575595,0],[0.612794,3.1358,0])
middleWingPos = #([2.61219,0.388254,0], [2.55568,1.7206,0], [2.51531,3.13373,0] )
IndexFingerPos = #([5.83321,-0.808869,0], [6.585,1.13922,0], [6.98073,3.10143,0])
clawPos = #([5.75,-1.13219,0], [5.86642,-1.6547,0], [5.95522,-2.00193,0], [6.10057,-2.29262,0])

BodyBones 			= createBones "Body" 		bodyPos 
ArmBones 			= createBones "Arm" 		armPos 				parentBone:BodyBones[1]
middleWingBones 	= createBones "MidWing" 	middleWingPos 	parentBone:ArmBones[2]
IndexFingerBones = createBones "Finger" 	IndexFingerPos 	parentBone:ArmBones[3]
clawBones 			= createBones "Claw" 		clawPos 			parentBone:ArmBones[3]
