/*----------------------------------------------------------------------------------------------------------------------------------------------------------
-- Wing rigger
-- By ken turner
-- www.designimage.co.uk
----------------------------------------------------------------------------------------------------------------------------------------------------------
-- This script will help you create a simple wing rig for bats or birds.
-- It has been smplified somewhat from the full production version.
-- Before reading through the script watch the video tutorial.
-- Once you understand what it's creating, it will be easier to understand how it works.

------------------------------------------------ using the script ---------------------------------------------------------------------------------------
-- Initially by hand you must create a few the 'master bones' the animator will use. 
-- Then you can use the script to quickly create the rest of the rig.
-- Use the script to outline the trailing edge with a nurbs spline .
-- Generate 'span bones' using the script. These automatically contol the shape of the wing between the master bones.
-- These span bones move using a collection of helpers and custom animation contollers all of which are created using the script.

*/----------------------------------------------------------------------------------------------------------------------------------------------------------
Rollout SplineRoll "Create wing splines"
(
	Local TheSpline -- This will be the NURBS spline that controls the wing shape. Its created using the createSpline function below
	Local hList = #() -- This will be a list of 4 helper objects used to control the spline shape via a splineIK modifier.
	Local StartBone , EndBone -- The user selects these two bones. The spline is created between thems
	
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	---------------------- Build the user Interface -------------------------------------------------------------------------------------------------------
	hyperlink hyp "www.designimage.co.uk" address:"www.designimage.co.uk" color:(color 125 125 255) align:#center 
	
	EditText EdTxt "Name" Text:"SplineIk"
	Checkbutton ChkBtnPickBones "Pick two Finger Bones"  width:250
	spinner SpSize "helper size" width:70 offset:[0,10] scale:0.1 type:#worldunits
	group "Start and end points"
	(
		Label lblStart "" Across:2
		Label lblEnd "" 
		
		RadioButtons RbtnStart "" labels:#("x","y","z") default:2  Across:2
		RadioButtons RbtnEnd "" labels:#("x","y","z") default:2 
		
		CheckBox ChkFlipStart "flip" Across:2
		CheckBox ChkFlipEnd "flip" 
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	/*------------------------ Functions ----------------------------------------------------------------------------------------------------------------------
	-- In MAXscript all function need to be defined before the can be used. All the functions you see near the top of the script are actually called by functions lower down.
	-- The main function in this rollout is called 'createSpline' which is in turn called by pressing the "Pick two Finger Bones" button
	-- Start down at 'On ChkBtnPickBones changed val do ...'  and follow the path of function calls moving up the script
	-- Tip: If you wish to quickly find a function 'ctrl right click' in the script window. This will bring up a list of functions to jump to.
	-- Tip: If you wish to quickly find a word use 'ctrl F' to find it.
	-- Tip: use 'ctrl+F2' to add bookmark. Then press 'F2' to quickly cycle through the code.
	*/
	fn NameIt =
	(
		-- Sensible object names are important for keeping track of complex rigs
		-- We'll rename objects as we create them.
		if TheSpline != undefined do 
		(
			TheSpline.name = EdTxt.text 

			hList[1].name = EdTxt.text + "_Start"
			hList[2].name = EdTxt.text + "_Start_tangent"
			hList[3].name = EdTxt.text + "_End_tangent"
			hList[4].name = EdTxt.text + "_End"
		)
	)

	/*--------------------- Script controller functions -----------------------------------------------------------------------------------------------------
	-- The wing rig is based on nurbs CV splines. The Splines outline a smooth edge for the wing surface.
	-- To stop the edge twisting. We'll keep the distance between each CV point on the spline roughly them same.
	-- As the wing closes the CV points to move closer together .
	-- As the wing opens, the CV points move further apart.
	-- To do this we simply split the distance between the end points into three
	-- The middle CV points go at 1/3rd and 2/3rds of this distance.
	
	-- We'll do this using a splineIK modifier attached to the spline. 
	-- Each of the 4 Curve CV points are moved by their own helper object.
	-- The position the middle two helper objects is automatically animated by script controllers
	-- The effect of this is easy to see in the example file.
	*/
	fn addScriptText theHelper axis flipped =
	(
		--This function creates the line of text used by the script controllers that controls the Spline shape.
		-- It simply divides the distance between two points by 3
	
		txt = "( distance p1.transform.pos p2.transform.pos / 3.0 )"
		
		-- The user can chose which axis the script controls.
		-- By changing the axis and flipped variables the user can make sure the spline points in the right direction
		
		if flipped do ( txt = "-" + txt ) -- We can do this since 'flipped' will either be 'true' or 'false' 
		
		-- text line is inserted into a point 3 array to give the X Y Z position of the helper retative to it's parent
		txt = case axis of
		(
			1:("[" + txt + " ,0,0]") -- place text in x axis
			2:("[0, " + txt + " ,0]") -- place text in y axis
			3:("[0,0, " + txt + "]") -- place text in z axis
		)
		
		-- Lastly we write the completed position script into the helpers script controller
		theHelper.position.controller.script = txt
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	-- A long time ago I was taught that if you do something more than once you need to make a function for it.
	-- These next two little functions package all the information needed to create the text for the script controller.
	-- They are called: when you create the spline, or if you change the splines direction ( by clicking the check boxes or radio buttons ).
	
	fn setStartAxis =
	(
		addScriptText hList[2] RbtnStart.state ChkFlipStart.checked
	)
	fn setEndAxis =
	(
		addScriptText hList[3] RbtnEnd.state ChkFlipEnd.checked
	)
	
	-- We can change an animation controller type using maxscript
	-- The next function changes the controller of 'theHelper'
	
	fn addScriptController theHelper =
	(
		p = position_script() -- Create a position script controller
		
		-- Tell the controller to reference the outer two helper objects. The helper objects are stored in an array called 'hlist'
		-- In the script controller this will be called 'p1' and 'p2'
		
		p.AddNode "p1" hList[1]
		p.AddNode "p2" hList[4]
		
		-- finally we change the controller of the helper object to our new script controller
		
		theHelper.position.controller = p
	)
	
	-------------------- Helper object placement -----------------------------------------------------------------------------
	fn placeAndLink theHelper obj = 
	(
		-- this function attached the ends of the spline to the bones picked by the user.
		-- If the bone has a child it will place the helper object at the child postion ( Hopefully the end of the bone ).
		-- If the bone has no child the spline is placed in the middle of the bone.
		-- Ultimatley the user is free to make adjustments after the spline is created simply by moving the helpers objects around.
		
		theHelper.position.controller.value = ( if obj.children.count == 0 then obj.center else obj.children[1].transform.position ) -- 'if' statements in max return a value
		theHelper.parent = obj
		theHelper.rotation.controller.value = (quat 0 0 0 1)  -- because the helper is now linked this aligns theHelpers rotation to its parent 
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	
	fn createSpline =
	(
		--The NURBS set holds all the data for the spline object we are creating 
		-- Once the created all the all the components of our curve we'll add them to the nurbs set
		-- Finally we use the NURBS set to create the object and add it to the scene
		
		nset = NURBSSet()

		-- create a new NURBS CV Curve, then setup the knots and CVs --------------------------------

		c = NURBSCVCurve name:"CV Curve" order:4 numCVs:4 numKnots:8
		for k in 1 to 4 do ( setKnot c k 0; setKnot c (k+4) 1 )
		cv = NURBSControlVertex [0, 0, 0]
		setCV c 1 cv
		cv.pos = [100, 0, 0]
		setCV c 2 cv
		cv.pos = [200, 0, 0]
		setCV c 3 cv
		cv.pos = [300, 0, 0]
		setCV c 4 cv
		
		-- add the NURBS CV Curve to the NURBS set
		appendObject nset c
		
		-- Create the NURBS object in the scene 
		TheSpline = NURBSNode nset name:EdTxt.text pos:[0,0,0]
		
		------------------------- This next part sets up the spline Ik ------------------------------------------------------------
		
		-- First create the modifier then add it to the nurbs curve
		-- linkTypes:2 means that the helper objects the modifer creates will be unlinked, ie have no parent ... We'll setup our own links shortly
		SpIkMod = Spline_IK_Control box:false helper_axistripod:true linkTypes:2
		addModifier TheSpline SpIkMod
		
		-- Using the modifier we create 4 helper objects to control the shape of the spline
		SpIkMod.createHelper 4
		
		redrawViews()  -- Update the screen and helper positions.
		
		hList = SpIkMod.helper_list
		hList[2].parent = hList[1]
		hList[3].parent = hList[4]
		placeAndLink hList[1] StartBone
		placeAndLink hList[4] EndBone

		addScriptController hList[2]
		addScriptController hList[3]
		
		setStartAxis()
		setEndAxis()

		
		NameIt() -- name all the objects we've just created 
			
		----------  update the user interface -------------------------------------------------------------------------------	
		lblStart.text = StartBone.name
		lblEnd.text = EndBone.name
		
		-- 
		-- if the 'helper size' spinner is set to zero then set a helper size based on the length of the spline. 
		if SpSize.value == 0 then SpSize.value = (distance hList[1].transform.position hList[4].transform.position) /16
		SpIkMod.helper_size = SpSize.value 
	)
	
	---------------------------- User Interface functions ------------------------------------------------------------------------------------------------
	
	fn fltNotAgain obj = ( obj != startBone )  -- this function is used when picking the end bone to make sure the same object is not picked twice
	On ChkBtnPickBones changed val do
	(
		-- Here the user must pick two bones 
		-- We give the user 3 different bits of feedback on what they are doing
		-- 1. This check button remains down while they are picking objects
		-- 2. The bone objects are hlighted by selection (using ' select:true ')
		-- 3. The second bone picked has a dotted line conecting it to the first bone (using ' rubberBand:StartBone.center ')
		
		ChkBtnPickBones.enabled = false
		StartBone = pickObject count:1 select:true
		if StartBone != undefined then 
		(
			EndBone = pickObject count:1 rubberBand:StartBone.center select:true filter:fltNotAgain
			if EndBone != undefined do 
			(
				-- so that our users can undo this operation in one go we'll use undo context to wrap object creation function.
				-- this also means that our users will see "Create wing spline" when they right click on the undo button
				
				with undo label:"Create wing spline" on createSpline()
			)
		)
		ChkBtnPickBones.enabled = true
		ChkBtnPickBones.checked= false
	)
	
	On SplineRoll close do 
	(
		-- cancel the pick object function when the rollout is closed
		if ChkBtnPickBones.checked then max select 
	)

	on SpSize changed val do (  if TheSpline != undefined do TheSpline.'spline ik control'.helper_size = val )
	
	on RbtnStart changed val do (  if TheSpline != undefined do  setStartAxis() )
	on ChkFlipStart changed val do (  if TheSpline != undefined do  setStartAxis() )
		
	on RbtnEnd changed val do (  if TheSpline != undefined do setEndAxis() )
	on ChkFlipEnd changed val do (  if TheSpline != undefined do setEndAxis() )
	on EdTxt entered val do nameIt()
)
----- End of SplineRoll ----------------------------------------------------------------------------------------------------------------------------------

/*-----------------------------------------------------------------------------------------------------------------------------------------------------------
This next section of MAXscript deals with the creation of the 'Span bones'.

It contains 3 structures definitions: BoneStruct, SplineStuct, and SpanBoneStruct.
Each structure contains a set of varibles and a set of functions that use those varibles.

-- This is a bit like creating simple geometry using MAXscript.
E.g to create a box you can type.
myBox = box()

Once the box has been created you can access it properties using a 'dot'
E.g. to set the boxes height type.
myBox.Height = 10

Here 'box' is a 'class definition'. A description of how to create all boxes in max.
The 'Class definition' says that all boxes must have certain properties like height, position, colour etc. 

'myBox' is a unique 'instance' of a 'box'. It gives all these properities a value... like myBox.height = 10

-- To create a new 'instance' of a structure simply call the 'structure definition' and assign it to a varible
E.g.
myBoneInstance = BoneStruct()

-- You can access varibles and functions in each 'instance' by using a 'dot'

E.g. to set a varible.
myBoneInstance.myNode = < some bone object in the scene >

E.g. to call a function
myBoneInstance.UpdateStartEnd False

-- Tip: when you start needing arrays of arrays its usually easier to create a structure definition
*/
----------------------------------------------------------- structures -----------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------

-- BoneStruct -----------------------------------------------------------------------------------------------------------------------------
-- Holds the root bone. -----------------------------------------
/*
	BoneStruct and splineStruct instances are held in the same 'splineList' array. (see the SpanRoll rollout below)
	Because they share function names it makes it easier to iterate through this array collecting and setting values
	For example:
		Both structures have the same function call 'getPos i'. 
		in BoneStruct:- This function returns a position along the length of the bone.
		in SplineStruct:-This function returns the position of the i'th helper object.
	
		Both functions return a position but they create that position in different ways.
*/
struct BoneStruct
(
	myNode,
	Flip = False,
	NumberOfHelpers = 0,
	startPos = [0,0,0],
	EndPos = [0,0,0],
	
	fn UpdateStartEnd FlipStartEnd = -- note that this function is does not exist in SplineStruct
	(
		flip = FlipStartEnd
		startPos = myNode.transform.Position
		EndPos = myNode.children[1].transform.Position
		if flip then 
		(
			startPos = EndPos
			EndPos = myNode.transform.Position
		)
	),
	fn GetPos i =
	(
		ratio = ( NumberOfHelpers + 1 ) as float
		w  = i / ratio
		(EndPos * w) + ((1-w) * startPos)
	),
	fn CreateHelpers n helperSize =
	(
		NumberOfHelpers = n
	),
	fn DestroyHelpers = 
	( 
		NumberOfHelpers = 0
	),
	fn setHelperSize val = 
	(
		-- dummy function helps Bonestruct to pretend to be a splineStruct
	),
	fn getHelper i = myNode
)
-- End of BoneStruct -----------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------------

-- SplineStuct -----------------------------------------------------------------------------------------------------------------------------
-- holds an array of helper objects used as 'look at targets' by the span bones -----------------------------------------

struct splineStruct
(
	myNode,
	lookAtHelperList = #(), -- A list of point helper objects attached to the spline using a Path constraint

	fn getPos i = lookAtHelperList[i].transform.position ,
	fn getHelper i = 	lookAtHelperList[i],
	fn CreateHelpers n helperSize = 
	(
		-- Create an array of point helpers and attach them to theSpline using a path constraint
		lookAtHelperList = for i = 1 to n collect 
		(
			p = point()  -- creates a point helper
			p.name = ( myNode.name + "_target_" + ( i as string ) )
			p.size = helperSize
			pc = Path_Constraint()
			pc.follow = true
			pc.axis  = 2  -- set the z axis of the helper to point along the spline
			pc.path = myNode
			
/*			-- Tip: You need to be careful when dividing integers. 
			-- Try typing  a = 1/2 into the listener
			-- Because both 1 and 2 are intergers the answer is also an integer. Thus 'a = 1/2 = 0' not '0.5' as we might expect.
			-- If the expression includes a floating point number all integers are converted to floating points before maths is calculated.
			-- If we had typed 'a = 1 / 2.0' then we'd get 'a = 0.5'. 
			-- Here both i and n are integers. We need to make sure they are converted to floats before the maths is calculated.
			-- This is why I've written: 'i * 100.0 / (n + 1.0)'.    Not:  'i * 100 / (n + 1)'.
*/
			deleteKeys pc.percent.controller.keys #allKeys
			pc.percent = i * 100.0 / (n + 1.0) -- a bit of maths to distribute the helpers along the spline
			
			p.position.controller = pc -- attach the point helper to the spline using the path constraint
			p -- Returns the point helper object -- This is 'collected' into the array 'lookAtHelperList'
		)
	),
	
	fn DestroyHelpers =
	(
		-- Before we try and delete an object, we'll check to see it still exists using 'isValidNode'. If we try and delete an object which doesn't exist MAXscript will thow an error.
		for obj in lookAtHelperList where isValidNode obj do delete obj
		lookAtHelperList = #()		
	),
	
	fn setHelperSize val = ( for obj in lookAtHelperList do obj.size = val ) -- Called when the user adjusts the helper size spinner 
)
-- End of SplineStruct -----------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------------

-- SpanBoneStruct ----------------------------------------------------------------------------------------------------------------------------------
-- holds a single chain of bones and set of functions to create or destroy them ----------------------------------------------------------------

struct SpanBoneStruct
(
	boneList = #(), -- a chain of bones
	RootParent = undefined, -- Used in bone edit mode
	fn SetLookAtControllers objList =
	(
		-- objList is an array of objects
		
		-- link the root bone to the first object in the list
		RootParent = boneList[1].parent = objList[1]
		
		-- use the remaining objects as 'look at targets'
		for i = 1 to (objList.count - 1) do
		(
			targetObjectIndex = i + 1
			boneList[i].rotation.controller = lookAt_constraint()
			boneList[i].rotation.controller.appendTarget objList[ targetObjectIndex ] 100
			boneList[i].rotation.controller.lookAt_vector_length=0
			boneList[i].rotation.controller.upNode_world = false
			boneList[i].rotation.controller.pickUpNode = objList[ targetObjectIndex ]
		)
	),

	fn SetBoneColor boneColor = for obj in boneList where isValidNode obj do obj.wirecolor = boneColor,
	fn SetBoneSize sz = 
	(
		for obj in boneList where isValidNode obj do obj.width = obj.height = sz
		boneList[boneList.count].length = sz
	),
	fn SetBoneNames	bonePrefix =
	(
		for i = 1 to boneList.count do
		(
			-- give the bone a sensible name
			boneList[i].name = if ( i == boneList.count) then ( bonePrefix + "End" ) else ( bonePrefix + (i as string) )
		)
	),

	fn CreateSpanBones posList boneColor BoneSize =
	(
		-- poslist is an array of positions. The start and end points of each bone.
		
		-- add a new position to the array to create the 'end bone' of the chain 
		v = posList[posList.count]  - posList[posList.count - 1] -- make a vector from the last two positions
		v = v /  ( length v ) -- normalise it
		endPoint = posList[posList.count] + ( v * BoneSize ) -- scale it by BoneSize and add to it the end
		append posList endPoint 
		lastBone = Undefined
		boneList = for i = 1 to ( posList.count - 1 ) collect
		(
			theBone = BoneSys.createBone posList[i] posList[i+1] [0,0,1]
			 
			theBone.wirecolor = boneColor
			theBone.width = theBone.height = BoneSize
			nodeInvalRect theBone -- make sure max view ports are refreshed where the bone is created
			theBone.parent = lastBone --  -- it's ok to set theBone.parent = undefined
			lastBone = theBone
			
			theBone -- return the bone for collection into the bonelist array
		)
	),
	fn DestroyBones =
	(
		-- delete all the bones and empty the bone list
		for obj in boneList where isValidNode obj do delete obj
		boneList = #()		
	),
	fn SetRootparent inEditMode =
	(
		If inEditMode then 
		(
			RootParent = boneList[1].parent  -- store the parent bone
			boneList[1].parent = undefined  -- unlink the  bone
		)else
		(
			boneList[1].parent = RootParent  -- relink the bone
		)
	)
)
---- End of SpanBoneStruct ------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------------------

rollout SpanRoll "Create Span bones"
(
	
	Local splineList = #() -- We'll fill this with an array of splineStruct instances
	Local SpanBoneList = #() -- We'll fill this with an array of SpanBoneStruct instances
	
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	---------------------- Build the user Interface -------------------------------------------------------------------------------------------------------
	
	Checkbutton ChkBtnPickSplines "Pick root bone then splines. Right click to finish" 
	
	spinner SpHelperSize "helper size" range:[0,200,5] Fieldwidth:50 type:#worldunits Align:#center
	
	Group "Root Bone"
	(
		Label LblRootBone "" 
		CheckBox chkReverse "reverse" Align:#right offset:[0,-22]
	)
	
	group "span Bones"
	(
		EditText EdTxt "Prefix" 
		Spinner SpBoneCount "Number of spans" type:#integer range:[0,50,2]  Fieldwidth:30 across:2
		spinner SpBoneSize "Size" range:[0,200,5] Fieldwidth:50 type:#worldunits 
		
		checkbutton btnBoneEdit "Bone Edit Mode"  across:2
		colorpicker ClrPckBone "Colour"   offset:[50,0]
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	------------------------ Functions ----------------------------------------------------------------------------------------------------------------------
	fn nameBones =
	(
		for i = 1 to SpanBoneList.count do
		(
			boneNamePrefix = EdTxt.text + "_span_" + ( i as string ) + "_bone_"
			SpanBoneList[i].SetBoneNames boneNamePrefix 
		)
	)
	fn createSpanBones =
	(
		splineList[1].UpdateStartEnd chkReverse.state
		n = SpBoneCount.value
		SpanBoneList = for i = 1 to n collect 
		(
			sp = SpanBoneStruct() -- create a structure instance to hold all the data.
			
			posList = for s in splineList collect s.getPos i  -- make an array of positions. Then use this array to create the bone chain.
			sp.CreateSpanBones posList ClrPckBone.color SpBoneSize.value 
				
			objList = for s in splineList collect s.getHelper i --make a new array by picking an object from each splineStruct and boneStruct
				
			sp.SetLookAtControllers objList -- set the bone chain parent and add 'look at' controllers using this array
			
			sp  -- return the structure instance to be collected in the SpanBoneList array
		)
		nameBones()
	)
	
	fn Addhelpers = for s in splineList do s.CreateHelpers SpBoneCount.value SpHelperSize.value
		
	fn RemoveHelpers = for s in splineList do s.DestroyHelpers()
	
	fn RemoveBones = for s in SpanBoneList do s.DestroyBones()
	
	fn UpdateSpanBones =
	(
		-- simply delete everything and start again
		-- This 'brute force' method is fast enough upto about 100 bones 		

		RemoveBones()
		RemoveHelpers()

		if SpBoneCount.value > 0 then 
		(
			Addhelpers()
			createSpanBones()
		)
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	---------------------------- User Interface functions ------------------------------------------------------------------------------------------------	
	on chkReverse changed state do 
	( 
		if splineList.count > 1 do UpdateSpanBones()
	)
	
	on EdTxt entered val do nameBones()
	
	on btnBoneEdit changed inEditMode do
	(
		if inEditMode and ( isCreatingObject Bones ) do StopCreating()
		maxops.affectchildren = ( not inEditMode )  -- When false the user can move bones without effecting the child bones.
		for s in SpanBoneList do s.SetRootparent inEditMode
	)
	
	on SpanRoll close do 
	(
		-- Setting 'maxops.affectchildren = false' freezes the auto key buttons. We need to make sure we un-freeze them when the rollout is closed
		maxops.affectchildren = true
		-- cancel the picking function
		if ChkBtnPickSplines.state then max select
	)
	-- Filter functions --------------------------------------------------------------------------------------------------------------------------------------------
	-- While the user is picking splines these functions filter out unwanted objects.
	-- Give them and object and they simply return 'true' or 'false'
	
	fn NotAlreadyPicked obj = 
	(
		test = true
		for s in splineList where s.myNode == obj while test do test = false
		test 
	)
	
	-- Check to see if the object is a spline that has not been clicked on before.
	fn goodSpline obj = ( (classof obj == NURBSCurveshape ) or ( classof obj == line ) ) and ( NotAlreadyPicked obj )
	----------------------------------------------------------------------------------------------------------------------------------------------------------
	
	On ChkBtnPickSplines changed val  do 
	(
		ChkBtnPickSplines.enabled = false -- stop the user clicking on this button until they've finished picking splines.
		theRoot = pickObject count:1 select:true
		
		if theRoot != undefined then 
		(
			EdTxt.text = LblRootBone.text = theRoot.name 
			
			splineList = #( boneStruct theRoot ) --  a boneStruct 'instance' containing the root bone.	
			-- now the user has picked a root bone we'll start to fill 'splineList' with 'splineStruct' instances.
			
			theSpline = pickObject count:1 select:true  rubberBand:(theRoot.center)  filter:goodSpline -- the filter option makes sure the user can only select 'valid' splines 
			while ( theSpline != undefined ) do 
			(
				append splineList (splineStruct theSpline)
				theSpline = pickObject count:1 rubberBand:(theSpline.center) select:true filter:goodSpline 
			)
			
			if splineList.count > 1 then -- if the user has selected a bone and some splines then we create the span bones
			(
				addhelpers()
				createSpanBones()
			)else -- otherwise clear everything
			(	
				splineList = #()
				SpanBoneList = #()
				EdTxt.text = LblRootBone.text = ""
			)
			
		)
		ChkBtnPickSplines.enabled = true
		ChkBtnPickSplines.checked= false
	)
	on ClrPckBone changed newBoneCol do 
	(
		-- update bone colour
		for s in SpanBoneList do s.SetBoneColor newBoneCol
	)
	on SpBoneCount changed val do 
	(
		-- check to see if the user has picked any splines first
		if splineList.count > 1 then UpdateSpanBones()
	)
	on SpBoneSize changed val do ( for s in SpanBoneList do s.SetBoneSize val )
	on SpHelperSize changed val do ( for s in splineList do s.setHelperSize val )
)
---- End of SpanRoll -----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------

-- Make sure we close old rollout floaters before we create a new one.
if WingRigFloater != undefined do ( closerolloutfloater WingRigFloater )
WingRigFloater = NewRolloutFloater "Wing Rigger" 300 440

addRollout SplineRoll WingRigFloater
addRollout SpanRoll WingRigFloater